@@ -257,7 +257,6 @@ sub _templateLog {
 #--------- Tripletail::DB専用
 sub _dbLog {
 	my $this = shift;
-	my $opts = { @_ };
 
 	if (!$this->{enabled}
 	|| (!$this->{db_popup} && !$this->{db_logging})
@@ -265,6 +264,15 @@ sub _dbLog {
 		return $this;
 	}
 	
+	my $opts;
+	if( @_ && UNIVERSAL::isa($_[0], 'CODE') )
+	{
+		my $coderef = shift;
+		$opts = { $coderef->(@_) };
+	}else
+	{
+		$opts = { @_ };
+	}
 	local($@);
 	
 	eval 'require Data::Dumper';
@@ -315,7 +323,6 @@ sub _dbLog {
 
 sub _dbLogData {
 	my $this = shift;
-	my $opts = { @_ };
 
 	if (!$this->{enabled}
 	|| (!$this->{db_popup} && !$this->{db_logging})
@@ -324,6 +331,16 @@ sub _dbLogData {
 		return $this;
 	}
 
+	my $opts;
+	if( @_ && UNIVERSAL::isa($_[0], 'CODE') )
+	{
+		my $coderef = shift;
+		$opts = { $coderef->(@_) };
+	}else
+	{
+		$opts = { @_ };
+	}
+	
 	my $dump;
 	my $data;
 	if(ref($opts->{data}) eq 'ARRAY') {
